from functools import wraps
from itertools import count

from matplotlib import pyplot as plt

from .contaxes import AxesContainer
from .contplot import PlotContainer

from .mpltypes import is_plot_artist
from .utils import iflatten, unseq


class AniPlot:

    def __init__(self, blit=True):
        self.blit = blit
        self.plots = set()
        self.axes = set()

        # handle resizing the window
        self.fig = fig = plt.gcf()
        fig.canvas.mpl_connect("draw_event", self.on_resize)


    def on_resize(self, event):
        self._save_bkgs()


    def __getattr__(self, name):
        attr = getattr(plt, name)

        if not callable(attr):
            return attr

        @wraps(attr)
        def wrapper(*args, **kwargs):
            res = attr(*args, **kwargs)

            plots = set()
            for obj in iflatten(res):
                if is_plot_artist(obj):
                    plots.add(obj)
                else:
                    print(f"ignoring: {name}() -> {obj}")

            if not plots:
                return res

            ax = plt.gca()
            plots = [PlotContainer(i, ax) for i in plots]

            fig = plt.gcf()
            ax = AxesContainer(ax, fig)

            self.plots.update(plots)
            self.axes.add(ax)

            return unseq(plots)

        return wrapper


    def show(self):
        fig = plt.gcf()
        canvas = fig.canvas
        canvas.draw()

        if self.blit:
            self._enable_animated_all()
            self._save_bkgs()

        plt.show(block=False)

        for i in count():
            yield i
            if self.blit:
                self._update_with_blit()
            else:
                canvas.draw()
            canvas.flush_events()


    def _update_with_blit(self):
        self._restore_bkgs()
        self._draw_all()
        self._blit_all()


    def _save_bkgs(self):
        for a in self.axes:
            a._save_bkg()

    def _restore_bkgs(self):
        for a in self.axes:
            a._restore_bkg()


    def _draw_all(self):
        for p in self.plots:
            p._draw()

    def _blit_all(self):
        for a in self.axes:
            a._blit()


    def _enable_animated_all(self):
        for p in self.plots:
            p._enable_animated()

    def _disable_animated_all(self):
        for p in self.plots:
            p._disable_animated()


    def enable_blit(self):
        self.blit = True
        self._enable_animated_all()

    def disable_blit(self):
        self.blit = False
        self._disable_animated_all()



